package com.starmicronics.mpopsdk;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.TextView;

import com.starmicronics.starioextension.starioextmanager.StarIoExtManager;
import com.starmicronics.starioextension.starioextmanager.StarIoExtManagerListener;

@SuppressWarnings("SynchronizeOnNonFinalField")
public class CashDrawerExtFragment extends Fragment {
    private ProgressDialog mProgressDialog;

    private TextView mComment;

    private StarIoExtManager mStarIoExtManager;

    private int mSelectedIndex;

    private boolean mIsFirst = true;

    public CashDrawerExtFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mProgressDialog = new ProgressDialog(getActivity());

        mProgressDialog.setMessage("Communicating...");
        mProgressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        mProgressDialog.setCancelable(false);

        PrinterSetting setting = new PrinterSetting(getActivity());

        mStarIoExtManager = new StarIoExtManager(StarIoExtManager.Type.Standard, setting.getPortName(), setting.getPrinterType(), 10000, getActivity());     // 10000mS!!!
        mStarIoExtManager.setListener(mStarIoExtManagerListener);

        Intent intent = getActivity().getIntent();
        mSelectedIndex = intent.getIntExtra(CommonActivity.BUNDLE_KEY_SELECTED_INDEX, 0);
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_cash_drawer_ext, container, false);

        mComment = (TextView) rootView.findViewById(R.id.statusTextView);

        Animation animation = AnimationUtils.loadAnimation(getActivity(), R.anim.blink);

        mComment.startAnimation(animation);

        Button testButton = (Button) rootView.findViewById(R.id.testOpenButton);

        testButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                sendCommands();
            }
        });

        setHasOptionsMenu(true);

        return rootView;
    }

    @Override
    public void onResume() {
        super.onResume();

        starIoExtManagerConnect();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.reloadIcon) {
            starIoExtManagerConnect();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void starIoExtManagerConnect() {
        AsyncTask<Void, Void, Boolean> asyncTask = new AsyncTask<Void, Void, Boolean>() {
            @Override
            protected void onPreExecute() {
                mProgressDialog.show();
            }

            @Override
            protected Boolean doInBackground(Void... params) {
                mStarIoExtManager.disconnect();
                return mStarIoExtManager.connect();
            }

            @Override
            protected void onPostExecute(Boolean result) {
                if (result) {
                    if (mIsFirst) {
                        mIsFirst = false;
                        sendCommands();
                    }
                    else {
                        mProgressDialog.dismiss();
                    }
                }
                else {
                    mProgressDialog.dismiss();
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(getActivity());

                    dialogBuilder.setTitle("Communication Result");
                    dialogBuilder.setMessage("failure.\nPrinter is offline.");
                    dialogBuilder.setPositiveButton("OK", null);

                    dialogBuilder.show();
                }

            }
        };

        asyncTask.execute();
    }

    @Override
    public void onPause() {
        super.onPause();

        mStarIoExtManager.disconnect();
    }

    @Override
    public void onStop() {
        super.onStop();

        if (mProgressDialog != null) {
            mProgressDialog.dismiss();
        }
    }

    private void sendCommands() {
        AsyncTask<Boolean, Void, Communication.Result> task = new AsyncTask<Boolean, Void, Communication.Result>() {
            @Override
            protected void onPreExecute() {
                mProgressDialog.show();
            }

            @Override
            protected Communication.Result doInBackground(Boolean... params) {
                byte[] data;
                Boolean doCheckCondition = params[0];

                switch (mSelectedIndex) {
                    default:
                        data = PrinterFunctions.createCommandsOpenCashDrawer();
                        break;
                }

                Communication.Result result;

                synchronized (mStarIoExtManager) {
                    if (doCheckCondition) {
                        result = Communication.sendCommands(data, mStarIoExtManager.getPort(), getActivity());
                    }
                    else {
                        result = Communication.sendCommandsDoNotCheckCondition(data, mStarIoExtManager.getPort(), getActivity());
                    }

                }

                return result;
            }

            @Override
            protected void onPostExecute(Communication.Result result) {
                if (isDetached() || getActivity() == null) return;

                if (mProgressDialog != null) {
                    mProgressDialog.dismiss();
                }

                String msg;

                switch (result) {
                    case Success:
                        msg = "Success!";
                        break;
                    case ErrorOpenPort:
                        msg = "Fail to openPort";
                        break;
                    case ErrorBeginCheckedBlock:
                        msg = "Printer is offline (beginCheckedBlock)";
                        break;
                    case ErrorEndCheckedBlock:
                        msg = "Printer is offline (endCheckedBlock)";
                        break;
                    case ErrorReadPort:
                        msg = "Read port error (readPort)";
                        break;
                    case ErrorWritePort:
                        msg = "Write port error (writePort)";
                        break;
                    default:
                        msg = "Unknown error";
                        break;
                }

                AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(getActivity());

                dialogBuilder.setTitle("Communication Result");
                dialogBuilder.setMessage(msg);
                dialogBuilder.setPositiveButton("OK", null);

                dialogBuilder.show();
            }
        };

        Boolean doCheckCondition;
        if (mSelectedIndex == 1) {
            doCheckCondition = true;
        }
        else {
            doCheckCondition = false;
        }

        task.execute(doCheckCondition);
    }

    private final StarIoExtManagerListener mStarIoExtManagerListener = new StarIoExtManagerListener() {
        @Override
        public void didPrinterImpossible() {
            mComment.setText("Printer Impossible.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didPrinterOnline() {
            mComment.setText("Printer Online.");

            mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didPrinterOffline() {
            mComment.setText("Printer Offline.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didCashDrawerOpen() {
            mComment.setText("Cash Drawer Open.");

            mComment.setTextColor(Color.MAGENTA);
        }

        @Override
        public void didCashDrawerClose() {
            mComment.setText("Cash Drawer Close.");

            mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didAccessoryConnectSuccess() {
            mComment.setText("Accessory Connect.");

            mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didAccessoryConnectFailure() {
            mComment.setText("Check the device.(Power and Bluetooth pairing)\nThen touch up the Refresh button.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didAccessoryDisconnect() {
            mComment.setText("Accessory Disconnect.");

            mComment.setTextColor(Color.RED);
        }
    };
}
