package com.starmicronics.mpopsdk;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.ListView;

public class CashDrawerFragment extends ItemListFragment {
    private ProgressDialog mProgressDialog;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mProgressDialog = new ProgressDialog(getActivity());

        mProgressDialog.setMessage("Communicating...");
        mProgressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        mProgressDialog.setCancelable(false);

        addTitle("Like a StarIO-SDK Sample");
        addMenu("BEL (Check condition.)");
        addMenu("BEL (Do not check condition. (F/W 1.1 or later.))");

        addTitle("StarIoExtManager Sample");
        addMenu("BEL (Check condition.)");
        addMenu("BEL (Do not check condition. (F/W 1.1 or later.))");
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);

        if (position <= 2) {

            AsyncTask<Boolean, Void, Communication.Result> task = new AsyncTask<Boolean, Void, Communication.Result>() {
                @Override
                protected void onPreExecute() {
                    mProgressDialog.show();
                }

                @Override
                protected Communication.Result doInBackground(Boolean... params) {
                    byte[]  data;
                    Boolean doCheckCondition = params[0];

                    data = PrinterFunctions.createCommandsOpenCashDrawer();

                    PrinterSetting setting = new PrinterSetting(getActivity());

                    Communication.Result result;

                    if (doCheckCondition) {
                        result = Communication.sendCommands(data, setting.getPortName(), setting.getPrinterType(), 10000, getActivity());     // 10000mS!!!
                    }
                    else {
                        result = Communication.sendCommandsDoNotCheckCondition(data, setting.getPortName(), setting.getPrinterType(), 10000, getActivity());     // 10000mS!!!
                    }
                    return result;
                }

                @Override
                protected void onPostExecute(Communication.Result result) {
                    if (isDetached() || getActivity() == null) return;

                    if (mProgressDialog != null) {
                        mProgressDialog.dismiss();
                    }

                    String msg;

                    switch (result) {
                        case Success :
                            msg = "Success!";
                            break;
                        case ErrorOpenPort:
                            msg = "Fail to openPort";
                            break;
                        case ErrorBeginCheckedBlock:
                            msg = "Printer is offline (beginCheckedBlock)";
                            break;
                        case ErrorEndCheckedBlock:
                            msg = "Printer is offline (endCheckedBlock)";
                            break;
                        case ErrorReadPort:
                            msg = "Read port error (readPort)";
                            break;
                        case ErrorWritePort:
                            msg = "Write port error (writePort)";
                            break;
                        default:
                            msg = "Unknown error";
                            break;
                    }

                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(getActivity());

                    dialogBuilder.setTitle("Communication Result");
                    dialogBuilder.setMessage(msg);
                    dialogBuilder.setPositiveButton("OK", null);

                    dialogBuilder.show();
                }
            };

            Boolean doCheckCondition;
            if (position == 1) {
                doCheckCondition = true;
            }
            else {
                doCheckCondition = false;
            }

            task.execute(doCheckCondition);
        }
        else {
            startTestDrawerActivity(position - 3);
        }
    }

    private void startTestDrawerActivity(int index) {
        Intent intent = new Intent(getActivity(), CommonActivity.class);

        intent.putExtra("ActivityLayout", R.layout.activity_cash_drawer_ext);
        intent.putExtra(CommonActivity.BUNDLE_KEY_SHOW_HOME_BUTTON, true);
        intent.putExtra(CommonActivity.BUNDLE_KEY_SHOW_RELOAD_BUTTON, true);
        intent.putExtra(CommonActivity.BUNDLE_KEY_TEST_BUTTON_TEXT, "Open");
        intent.putExtra(CommonActivity.BUNDLE_KEY_SELECTED_INDEX,   index);

        startActivity(intent);
    }
}
