package com.starmicronics.mpopsdk;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import com.starmicronics.starioextension.commandbuilder.Bitmap.SCBBitmapConverter;
import com.starmicronics.starioextension.starioextmanager.StarIoExtManager;
import com.starmicronics.starioextension.starioextmanager.StarIoExtManagerListener;

import java.util.ArrayList;
import java.util.List;

@SuppressWarnings({"FieldCanBeLocal", "SynchronizeOnNonFinalField"})
public class CombinationExtFragment extends Fragment {
    private final int LIST_MAX_NUM = 30;

    private ProgressDialog mProgressDialog;

    private List<ItemList>  mList;
    private ItemListAdapter mAdapter;

    private StarIoExtManager mStarIoExtManager;

    private ListView mListView;
    private TextView mComment;

    private int mSelectedIndex;

    public CombinationExtFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mProgressDialog = new ProgressDialog(getActivity());

        mProgressDialog.setMessage("Communicating...");
        mProgressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        mProgressDialog.setCancelable(false);

        mList    = new ArrayList<>();
        mAdapter = new ItemListAdapter(getActivity(), mList);

        PrinterSetting setting = new PrinterSetting(getActivity());

        mStarIoExtManager = new StarIoExtManager(StarIoExtManager.Type.WithBarcodeReader, setting.getPortName(), setting.getPrinterType(), 10000, getActivity());     // 10000mS!!!

        mStarIoExtManager.setListener(mStarIoExtManagerListener);

        Intent intent = getActivity().getIntent();

        mSelectedIndex = intent.getIntExtra(CommonActivity.BUNDLE_KEY_SELECTED_INDEX, 0);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_combination_ext, container, false);

        mListView = (ListView) rootView.findViewById(R.id.bcrTestListView);

        mListView.setAdapter(mAdapter);

        mComment = (TextView) rootView.findViewById(R.id.bcrStatusTextView);

        Animation animation = AnimationUtils.loadAnimation(getActivity(), R.anim.blink);

        mComment.startAnimation(animation);

        Button testButton = (Button) rootView.findViewById(R.id.bcrTestButton);

        testButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                sendCommands();
            }
        });

        setHasOptionsMenu(true);

        return rootView;
    }

    @Override
    public void onResume() {
        super.onResume();

        starIoExtManagerConnect();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.reloadIcon) {
            starIoExtManagerConnect();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void starIoExtManagerConnect() {
        AsyncTask<Void, Void, Boolean> asyncTask = new AsyncTask<Void, Void, Boolean>() {
            @Override
            protected void onPreExecute() {
                mProgressDialog.show();
            }

            @Override
            protected Boolean doInBackground(Void... params) {
                mStarIoExtManager.disconnect();
                return mStarIoExtManager.connect();
            }

            @Override
            protected void onPostExecute(Boolean result) {
                mProgressDialog.dismiss();

                if (!result) {
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(getActivity());

                    dialogBuilder.setTitle("Communication Result");
                    dialogBuilder.setMessage("failure.\nPrinter is offline.");

                    dialogBuilder.setPositiveButton("OK", null);

                    dialogBuilder.show();
                }
            }
        };

        asyncTask.execute();
    }

    private void sendCommands() {
        AsyncTask<Void, Void, Communication.Result> task = new AsyncTask<Void, Void, Communication.Result>() {
            @Override
            protected void onPreExecute() {
                if (!mProgressDialog.isShowing()) {
                    mProgressDialog.show();
                }
            }

            @Override
            protected Communication.Result doInBackground(Void... params) {
                CommandDataList dataList = new CommandDataList();

                byte[] data;

                switch (mSelectedIndex) {
                    default:
                    case 1:
                        data = PrinterFunctions.createCommandsEnglish2inchLineModeReceipt();
                        break;
                    case 2:
                        data = PrinterFunctions.createCommandsEnglish2inchRasterModeReceipt(384);
                        break;
                    case 3:
                        data = PrinterFunctions.createCommandsEnglish3inchRasterModeReceipt(384, true);
                        break;
                    case 4:
                        data = PrinterFunctions.createCommandsEnglish3inchRasterModeReceipt(384, false);
                        break;
                    case 5:
                        data = PrinterFunctions.createCommandsEnglishRasterModeCoupon(384, SCBBitmapConverter.Rotation.Normal, getResources());
                        break;
                    case 6:
                        data = PrinterFunctions.createCommandsEnglishRasterModeCoupon(384, SCBBitmapConverter.Rotation.Right90, getResources());
                        break;
                }

                dataList.add(data);

                dataList.add(PrinterFunctions.createCommandsOpenCashDrawer());

                Communication.Result result;

                synchronized (mStarIoExtManager) {
                    result = Communication.sendCommands(dataList.getByteArray(), mStarIoExtManager.getPort(), getActivity());     // 10000mS!!!
                }

                return result;
            }

            @Override
            protected void onPostExecute(Communication.Result result) {
                if (isDetached() || getActivity() == null) return;

                if (mProgressDialog.isShowing()) {
                    mProgressDialog.dismiss();
                }

                String msg;

                switch (result) {
                    case Success:
                        msg = "Success!";
                        break;
                    case ErrorOpenPort:
                        msg = "Fail to openPort";
                        break;
                    case ErrorBeginCheckedBlock:
                        msg = "Printer is offline (beginCheckedBlock)";
                        break;
                    case ErrorEndCheckedBlock:
                        msg = "Printer is offline (endCheckedBlock)";
                        break;
                    case ErrorReadPort:
                        msg = "Read port error (readPort)";
                        break;
                    case ErrorWritePort:
                        msg = "Write port error (writePort)";
                        break;
                    default:
                        msg = "Unknown error";
                        break;
                }

                AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(getActivity());

                dialogBuilder.setTitle("Communication Result");
                dialogBuilder.setMessage(msg);
                dialogBuilder.setPositiveButton("OK", null);

                dialogBuilder.show();
            }
        };

        task.execute();
    }

    @Override
    public void onPause() {
        super.onPause();

        mStarIoExtManager.disconnect();
    }

    private void addItem(String text) {
        List<TextInfo> textList = new ArrayList<>();

        textList.add(new TextInfo(text, R.id.bcrValueTextView));
        mAdapter.add(new ItemList(R.layout.list_bcr_row, textList, true));

        if (mAdapter.getCount() > LIST_MAX_NUM) {
            mAdapter.remove(mAdapter.getItem(0));
        }

        int itemCount = mList.size();
        mListView.setSelection(itemCount - 1);
    }

    private final StarIoExtManagerListener mStarIoExtManagerListener = new StarIoExtManagerListener() {
        @Override
        public void didBarcodeDataReceive(byte[] barcodeData) {
            String[] barcodeDataArray = new String(barcodeData).split("\n");

            for(String data:barcodeDataArray) {
                addItem(data);
            }
        }

        @Override
        public void didPrinterImpossible() {
            mComment.setText("Printer Impossible.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didPrinterOnline() {
            mComment.setText("Printer Online.");

            mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didPrinterOffline() {
//          mComment.setText("Printer Offline.");
//
//          mComment.setTextColor(Color.RED);
        }

        @Override
        public void didPrinterPaperReady() {
//          mComment.setText("Printer Paper Ready.");
//
//          mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didPrinterPaperNearEmpty() {
            // This method is not called with mPOP.
            // Because mPOP does not have "Paper Near Empty sensor".
            mComment.setText("Printer Paper Near Empty.");

            mComment.setTextColor(0xffffa500);     // Orange
        }

        @Override
        public void didPrinterPaperEmpty() {
            mComment.setText("Printer Paper Empty.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didPrinterCoverOpen() {
            mComment.setText("Printer Cover Open.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didPrinterCoverClose() {
//          mComment.setText("Printer Cover Close.");
//
//          mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didCashDrawerOpen() {
            mComment.setText("Cash Drawer Open.");

            mComment.setTextColor(Color.MAGENTA);
        }

        @Override
        public void didCashDrawerClose() {
            mComment.setText("Cash Drawer Close.");

            mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didBarcodeReaderImpossible() {
            mComment.setText("Barcode Reader Impossible.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didBarcodeReaderConnect() {
            mComment.setText("Barcode Reader Connect.");

            mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didBarcodeReaderDisconnect() {
            mComment.setText("Barcode Reader Disconnect.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didAccessoryConnectSuccess() {
            mComment.setText("Accessory Connect.");

            mComment.setTextColor(Color.BLUE);
        }

        @Override
        public void didAccessoryConnectFailure() {
            mComment.setText("Check the device.(Power and Bluetooth pairing)\nThen touch up the Refresh button.");

            mComment.setTextColor(Color.RED);
        }

        @Override
        public void didAccessoryDisconnect() {
            mComment.setText("Accessory Disconnect.");

            mComment.setTextColor(Color.RED);
        }
    };
}
