package com.starmicronics.mpopsdk;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.TextView;

import java.util.List;

class ItemListAdapter extends ArrayAdapter<ItemList> {
    final private LayoutInflater mInflater;
    final private List<ItemList> mItems;

    public ItemListAdapter( Context context, List<ItemList> items)
    {
        super(context, 0, items);

        mItems    = items;
        mInflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        ItemList item = mItems.get(position);

        convertView = mInflater.inflate(item.getLayoutResourceID(), parent, false);

        List<TextInfo> textList = item.getTextList();

        for (TextInfo textInfo : textList) {
            TextView textView = (TextView) convertView.findViewById( textInfo.getResourceID() );
            textView.setText(textInfo.getText());

            if (textInfo.isCustomTextColor()) {
                textView.setTextColor(textInfo.getTextColor());
            }
        }

        return convertView;
    }

    @Override
    public boolean isEnabled(int position) {
        return mItems.get(position).isClickable();            // True: can click list.  False: cannot click list.
    }

}

class ItemList {
    private final int             mLayoutResourceID;
    private final List<TextInfo>  mTextList;
    private final boolean         mIsClickable;

    public ItemList(int layoutResourceID, List<TextInfo> textList, boolean isClickable) {
        this.mLayoutResourceID = layoutResourceID;
        this.mTextList         = textList;
        this.mIsClickable      = isClickable;
    }

    public int getLayoutResourceID() {
        return mLayoutResourceID;
    }

    public List<TextInfo> getTextList() {
        return mTextList;
    }

    public boolean isClickable() {
        return mIsClickable;
    }
}
