package com.starmicronics.mpopsdk;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.ListView;

import com.starmicronics.starioextension.commandbuilder.Bitmap.SCBBitmapConverter;

public class PrinterFragment extends ItemListFragment {
    private ProgressDialog mProgressDialog;

    private int mSelectedIndex;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mProgressDialog = new ProgressDialog(getActivity());

        mProgressDialog.setMessage("Communicating...");
        mProgressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        mProgressDialog.setCancelable(false);

        addTitle("Like a StarIO-SDK Sample");
        addMenu("En-Us 2\" Line Receipt");
        addMenu("En-Us 2\" Raster Receipt");
        addMenu("En-Us 3\" Raster Receipt (Both Scale)");
        addMenu("En-Us 3\" Raster Receipt (Scale)");
        addMenu("En-Us Raster Coupon");
        addMenu("En-Us Raster Coupon (Rotation90)");

        addTitle("StarIoExtManager Sample");
        addMenu("En-Us 2\" Line Receipt");
        addMenu("En-Us 2\" Raster Receipt");
        addMenu("En-Us 3\" Raster Receipt (Both Scale)");
        addMenu("En-Us 3\" Raster Receipt (Scale)");
        addMenu("En-Us Raster Coupon");
        addMenu("En-Us Raster Coupon (Rotation90)");
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);

        if (position <= 6) {
            mSelectedIndex = position;

            AsyncTask<Void, Void, Communication.Result> task = new AsyncTask<Void, Void, Communication.Result>() {
                @Override
                protected void onPreExecute() {
                    mProgressDialog.show();
                }

                @Override
                protected Communication.Result doInBackground(Void... params) {
                    byte[] data;

                    switch (mSelectedIndex) {
                        default:
                        case 1:
                            data = PrinterFunctions.createCommandsEnglish2inchLineModeReceipt();
                            break;
                        case 2:
                            data = PrinterFunctions.createCommandsEnglish2inchRasterModeReceipt(384);
                            break;
                        case 3:
                            data = PrinterFunctions.createCommandsEnglish3inchRasterModeReceipt(384, true);
                            break;
                        case 4:
                            data = PrinterFunctions.createCommandsEnglish3inchRasterModeReceipt(384, false);
                            break;
                        case 5:
                            data = PrinterFunctions.createCommandsEnglishRasterModeCoupon(384, SCBBitmapConverter.Rotation.Normal, getResources());
                            break;
                        case 6:
                            data = PrinterFunctions.createCommandsEnglishRasterModeCoupon(384, SCBBitmapConverter.Rotation.Right90, getResources());
                            break;
                    }

                    PrinterSetting setting = new PrinterSetting(getActivity());

                    Communication.Result result;

                    result = Communication.sendCommands(data, setting.getPortName(), setting.getPrinterType(), 10000, getActivity());     // 10000mS!!!

                    return result;
                }

                @Override
                protected void onPostExecute(Communication.Result result) {
                    if (isDetached() || getActivity() == null) return;

                    if (mProgressDialog != null) {
                        mProgressDialog.dismiss();
                    }

                    String msg;

                    switch (result) {
                        case Success:
                            msg = "Success!";
                            break;
                        case ErrorOpenPort:
                            msg = "Fail to openPort";
                            break;
                        case ErrorBeginCheckedBlock:
                            msg = "Printer is offline (beginCheckedBlock)";
                            break;
                        case ErrorEndCheckedBlock:
                            msg = "Printer is offline (endCheckedBlock)";
                            break;
                        case ErrorReadPort:
                            msg = "Read port error (readPort)";
                            break;
                        case ErrorWritePort:
                            msg = "Write port error (writePort)";
                            break;
                        default:
                            msg = "Unknown error";
                            break;
                    }

                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(getActivity());

                    dialogBuilder.setTitle("Communication Result");
                    dialogBuilder.setMessage(msg);
                    dialogBuilder.setPositiveButton("OK", null);

                    dialogBuilder.show();
                }
            };

            task.execute();
        }
        else {
            startTestPrintActivity(position - 7);
        }
    }

    private void startTestPrintActivity(int index) {
        Intent intent = new Intent(getActivity(), CommonActivity.class);

        intent.putExtra("ActivityLayout", R.layout.activity_printer_ext);
        intent.putExtra(CommonActivity.BUNDLE_KEY_SHOW_HOME_BUTTON, true);
        intent.putExtra(CommonActivity.BUNDLE_KEY_SHOW_RELOAD_BUTTON, true);
        intent.putExtra(CommonActivity.BUNDLE_KEY_TEST_BUTTON_TEXT, "Print");
        intent.putExtra(CommonActivity.BUNDLE_KEY_SELECTED_INDEX,   index);

        startActivity(intent);
    }
}
