package com.starmicronics.mpopsdk;

import android.app.ProgressDialog;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.ListView;
import android.widget.TextView;

import com.starmicronics.stario.PortInfo;
import com.starmicronics.stario.StarIOPort;
import com.starmicronics.stario.StarIOPortException;

import java.util.ArrayList;
import java.util.List;

public final class PrinterSearchFragment extends ItemListFragment {
    private ProgressDialog mProgressDialog;

    public PrinterSearchFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        addTitle("List");

        SearchTask btSearchTask = new SearchTask();

        btSearchTask.execute("BT:");

        SearchTask usbSearchTask = new SearchTask();

        usbSearchTask.execute("USB:");

        mProgressDialog = new ProgressDialog(getActivity());

        mProgressDialog.setMessage("Communicating...");
        mProgressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        mProgressDialog.setCancelable(false);
        mProgressDialog.show();
    }

    @Override
    public void onStop() {
        super.onStop();

        if (mProgressDialog != null) {
            mProgressDialog.dismiss();
        }
    }

    @Override
    public void onListItemClick(ListView l, View clickedItemView, int position, long id) {
        super.onListItemClick(l, clickedItemView, position, id);

        TextView deviceNameView = (TextView) clickedItemView.findViewById(R.id.deviceNameTextView);
        String   deviceName     = deviceNameView.getText().toString();

        TextView macAddressView = (TextView) clickedItemView.findViewById(R.id.macAddressTextView);
        String   macAddress     = macAddressView.getText().toString();

        registerPrinter(deviceName, macAddress);

        getActivity().finish();
    }

    private void addItem(PortInfo info) {
        List<TextInfo> textList = new ArrayList<>();

        String deviceName = info.getPortName();
        String macAddress = info.getMacAddress();

        // --- Bluetooth ---
        // It can communication used device name(Ex.BT:Star Micronics) at bluetooth.
        // If android device has paired two same name device, can't choose destination target.
        // If used Mac Address(Ex. BT:00:12:3f:XX:XX:XX) at Bluetooth, can choose destination target.
        if (deviceName.startsWith("BT:")) {
            deviceName = deviceName.substring("BT:".length());
            macAddress = "BT:" + macAddress;
        }

        textList.add(new TextInfo(deviceName, R.id.deviceNameTextView));
        textList.add(new TextInfo(macAddress, R.id.macAddressTextView));

        adapter.add(new ItemList(R.layout.list_printer_info_row, textList, true));
    }

    /**
     * Register printer information at SharedPreference.
     */
    private void registerPrinter(String deviceName, String macAddress) {
        PrinterSetting setting = new PrinterSetting(getActivity());

        setting.write(deviceName, macAddress);
    }

    /**
     * Printer search task.
     */
    public class SearchTask extends AsyncTask<String, Integer, Integer> {
        private List<PortInfo> mPortList;

        public SearchTask() {
            super();
        }

        @Override
        protected Integer doInBackground(String... interfaceType) {
            try {
                mPortList = StarIOPort.searchPrinter(interfaceType[0], getActivity());
            }
            catch (StarIOPortException e) {
                mPortList = new ArrayList<>();
            }

            return 0;
        }

        @Override
        protected void onPostExecute(Integer doNotUse) {
            for (PortInfo info : mPortList) {
                addItem(info);
            }

            if (mProgressDialog != null) {
                mProgressDialog.dismiss();
            }
        }
    }
}
